@@ -29,7 +29,7 @@ WEIGHT_FNS = {
 }
 ERROR_OPTIONS = ['fdr', 'local_fdr', 'fwer', 'pfer']
 BINARY_TOL = 1e-3
-DEFAULT_GRID_SIZES = np.around(np.logspace(np.log10(50), np.log10(4), 25))
+DEFAULT_GRID_SIZES = np.around(np.logspace(np.log10(4), 4, 25))
 
 def BLiP(
 	samples=None,
@@ -37,9 +37,10 @@ def BLiP(
 	weight_fn='inverse_size',
 	error='fdr',
 	q=0.1,
-	max_pep=0.5,
+	max_pep=0.25,
 	deterministic=True,
 	verbose=True,
+	solver_verbose=False,
 	perturb=True,
 	max_iters=100,
 	search_method='binary',
@@ -80,6 +81,8 @@ def BLiP(
 		If True, guarantees a deterministic solution. Defaults to True.
 	verbose : bool
 		If True, gives occasional progress reports.
+	solver_verbose : bool
+		If True, the underlying LP solver is verbose.
 	search_method : string
 		For FWER control, how to find the optimal parameter
 		for the LP. Either "none" or "binary." Defaults to "binary". 
@@ -216,7 +219,6 @@ def BLiP(
 	b = np.ones(nrel)
 	v_param = cp.Parameter()
 	v_param.value = q
-	v_var = cp.Variable(pos=True) # for FDR only
 
 	# We perform a binary for FWER to find optimal v.
 	if samples is not None and error == 'fwer' and search_method == 'binary':
@@ -238,10 +240,7 @@ def BLiP(
 	# Extra constraints for fdr
 	elif error == 'fdr':
 		constraints += [
-			x @ peps <= v_var,
-			cp.sum(x) >= v_var / q,
-			v_var >= 0,
-			v_var <= q * nrel + 1
+			x @ (peps - q) <= 0
 		]
 
 	# Create problem
@@ -250,7 +249,7 @@ def BLiP(
 
 	# Solve problem once if we do not need to search over v
 	if not binary_search:
-		problem.solve(solver=solver)
+		problem.solve(solver=solver, verbose=solver_verbose)
 		selections = x.value
 
 	# Solve FWER through binary search
@@ -263,7 +262,7 @@ def BLiP(
 			v_current = (v_upper + v_lower)/2
 			v_param.value = v_current 
 			# Solve
-			problem.solve(solver=solver, warm_start=True)
+			problem.solve(solver=solver, warm_start=True, verbose=solver_verbose)
 			selections = x.value
 
 			# Round selections---could do something smarter, TODO
@@ -284,7 +283,7 @@ def BLiP(
 
 		# Solve with v_lower for final solution
 		v_param.value = v_lower
-		problem.solve(solver=solver, warm_start=True)
+		problem.solve(solver=solver, warm_start=True, verbose=solver_verbose)
 		if problem.status == 'infeasible':
 			selections = np.zeros(ngroups)
 		else:
@@ -321,9 +320,11 @@ def BLiP_cts(
 	locs,
 	grid_sizes=DEFAULT_GRID_SIZES,
 	weight_fn=weight_fns.inverse_radius_weight,
+	extra_centers=None,
 	max_pep=0.25,
-	max_blip_size=1500,
-	rescale=True,
+	shape='circle',
+	min_blip_size=1500,
+	verbose=True,
 	**kwargs
 ):
 	"""
@@ -340,12 +341,26 @@ def BLiP_cts(
 		between posterior iterations.)
 	grid_sizes : list or np.ndarray
 		List of grid sizes to split up the locations.
-	rescale : bool
-		If True, normalizes locs to ensure they are all between 0 and 1.
-		If ``rescale`` is False and this is not true, the function
-		will error.
+	weight_fn : string or function
+		A function mapping ``CandidateGroup`` objects to nonnegative weights.
+		This defines the power function. Defaults to ``inverse_radius``.
+	extra_centers : np.ndarray
+		A (ncenters, d)-dimensional array. At each resolution,
+		candidate groups will be computed with centers at these
+		locations.
+	max_pep : float
+		The maximum allowable PEP for output candidate groups.
+		Defaults to 0.25.
+	shape : string
+		One of ``square`` or ``circle``
+	min_blip_size : int
+		Combines connected components so all subproblems are of 
+		at least this size.
+	verbose : bool
+		If True (default), gives occasional status updates.
 	kwargs : dict
-		Additional arguments to pass to the underlying BLiP call.
+		Additional arguments to pass to the underlying BLiP call,
+		such as the error rate or nominal level.
 
 	Returns
 	-------
@@ -358,26 +373,25 @@ def BLiP_cts(
 	"""
 
 	# Normalize locations
-	d = locs.shape[2] # dimensionality
-	if rescale:
-		norm_locs, shifts, scales = create_groups_cts.normalize_locs(locs)
-	else:
-		minval = locs[~np.isnan(locs)].min()
-		maxval = locs[~np.isnan(locs)].max()
-		if minval < 0 or maxval > 1:
-			raise ValueError(f"rescale=False but locs are not between 0 and 1")
-		norm_locs = locs.copy() 
-		shifts = np.zeros(d)
-		scales = np.ones(d)
+	N, d = locs.shape # dimensionality
+	log_interval = int(max(1, np.around(N/10))) if verbose else None
 
 	# 1. Calculate filtered PEPs
 	peps = create_groups_cts.grid_peps(
-		locs=norm_locs, grid_sizes=grid_sizes, max_pep=max_pep
+		locs=locs,
+		grid_sizes=grid_sizes,
+		extra_centers=extra_centers,
+		max_pep=max_pep,
+		shape=shape,
+		log_interval=log_interval,
 	)
 
 	# 2. Calculate nodes, components, and so on
 	all_cand_groups, _ = create_groups_cts.grid_peps_to_cand_groups(
-		peps, max_blip_size=max_blip_size
+		peps,
+		min_blip_size=min_blip_size,
+		shape=shape, 
+		max_pep=max_pep,
 	)
 
 	# 3. Run BLiP
@@ -391,14 +405,13 @@ def BLiP_cts(
 		)
 		all_rej.extend(rej)
 
-	# 4. Renormalize locations
+	# 4. Make locations easier to interpret
 	for cand_group in all_rej:
 		center = np.zeros(d)
-		radius = cand_group.data.pop('radius') * scales + shifts
 		for k in range(d):
-			center[k] = cand_group.data.pop(f'dim{k}') * scales[k] + shifts[k]
+			center[k] = cand_group.data.pop(f'dim{k}')
 		cand_group.data['center'] = center
-		cand_group.data['radii'] = radius
+		cand_group.data['radii'] = np.ones(d) * radius
 
 	return all_rej
 
@@ -462,13 +475,14 @@ def binarize_selections(
 
 	# Constraints to ensure selected groups are disjoint
 	nontriv_cand_groups, nrel = create_groups._elim_redundant_features(nontriv_cand_groups)
+	nontriv_cand_groups, nrel = create_groups._ecc_reduction(nontriv_cand_groups)
 	A = np.zeros((ngroups, nrel), dtype=bool)
 	for gj, cand_group in enumerate(nontriv_cand_groups):
 		for feature in cand_group.data['blip-group']:
 			A[gj, feature] = 1
 	if verbose:
 		msg = f"LP had {ngroups} non-integer solutions across {nrel} locations."
-		msg += f" Binarizing using deterministic={deterministic}."
+		msg += f" Binarizing with deterministic={deterministic}."
 		print(msg)
 
 	# Sampling method
@@ -577,16 +591,21 @@ def binarize_selections(
 			# Iteratively try to solve problem and then backtrack if infeasible
 			# (backtracking is extremely rare)
 			while len(output) >= 0:
-				v_var = cp.Variable(pos=True)
+				fdr_rhs = ndisc_out * q - v_output
 				constraints_fdr = constraints + [
-					x @ peps <= v_var,
-					cp.sum(x) >= (v_var + v_output) / q - ndisc_out
+					x @ (peps - q) <= fdr_rhs,
 				]
 				problem = cp.Problem(objective=objective, constraints=constraints_fdr)
 				try:
-					problem.solve(solver='GLPK_MI')
-				except cp.error.SolverError as e:
 					problem.solve(solver=BIN_SOLVER)
+					# Check that this yields a real feasible solution
+					if x.value is not None:
+						max_dist = np.minimum(np.abs(x.value), np.abs(x.value-1)).max()
+						if problem.status != 'infeasible':
+							if x.value @ (peps - q) > fdr_rhs + tol or max_dist > tol:
+								raise cp.error.SolverError(f"Solver={BIN_SOLVER} yielded infeasible solution") 
+				except cp.error.SolverError as e:
+					problem.solve(solver='GLPK_MI')
 				if problem.status != 'infeasible':
 					break
 				else:
