@@ -4,6 +4,8 @@ import numpy as np
 # Tree methods from scipy
 import scipy.cluster.hierarchy as hierarchy
 import scipy.spatial.distance as ssd
+import networkx as nx
+from .ecc import edge_clique_cover
 
 MIN_PEP = 1e-15 # for numerical stability
 
@@ -54,7 +56,7 @@ def all_cand_groups(
 	samples,
 	X=None,
 	q=0,
-	max_pep=1,
+	max_pep=0.25,
 	max_size=25,
 	prenarrow=True,
 	prefilter_thresholds=[0, 0.01, 0.02, 0.03, 0.05, 0.1, 0.2]
@@ -70,7 +72,7 @@ def all_cand_groups(
 		An ``(N, p)``-shaped array of posterior samples,
 		where a nonzero value indicates the presence of a signal.
 	X : np.array
-		The n x p deisgn matrix. Defaults to ``None.``
+		The n x p design matrix. Defaults to ``None.``
 		If provided, adds hierarchical groups based 
 		on a correlation cluster of X.
 	q : float
@@ -136,11 +138,13 @@ def all_cand_groups(
 
 def susie_groups(
 	alphas,
-	X,
 	q,
-	max_pep=1,
+	X=None,
+	max_pep=0.25,
 	max_size=25,
-	prenarrow=False
+	prenarrow=False,
+	purity_threshold=0.0,
+	k_threshold=None,
 ):
 	"""
 	Creates candidate groups based on a SuSiE fit.
@@ -151,12 +155,12 @@ def susie_groups(
 		An ``(L, p)``-shaped matrix of alphas from a SuSiE object,
 		``L`` is the number of SuSiE iterations and ``p``
 		is the number of covariates.
-	X : np.array
-		The n x p deisgn matrix. Defaults to ``None.``
-		If provided, adds hierarchical groups based 
-		on a correlation cluster of X.
 	q : float
 		The level at which to control the error rate
+	X : np.array
+		The n x p design matrix. Defaults to ``None.``
+		If provided, adds hierarchical groups based 
+		on a correlation cluster of X.
 	max_pep : float
 		The maximum posterior error probability allowed in
 		a candidate group. Default is 1.
@@ -165,6 +169,13 @@ def susie_groups(
 	prenarrow : bool
 		If true, "prenarrows" the candidate groups
 		as described in the paper. Defaults to False.
+	purity_threshold : float
+		When computing PIPs, ignores SuSiE iterations which
+		do not pass this purity threshold, as in the original
+		SusiE paper.
+	k_threshold : float
+		When computing PIPs, ignores SuSiE iterations for which
+		the vanilla SuSiE CS exceeds this size.
 
 	Returns
 	-------
@@ -173,7 +184,37 @@ def susie_groups(
 	"""
 	L, p = alphas.shape
 	np.random.seed(1)
-	# Start with sequential groups
+	# Preprocessing
+	if purity_threshold > 0 and X is None:
+		raise ValueError("When purity_threshold > 0, X must be provided.")
+	if k_threshold is None:
+		k_threshold = p + 1
+	if purity_threshold > 0:
+		Sigma = np.corrcoef(X.T)
+
+	# Add groups discovered by susie
+	Linds = []
+	groups_to_add = []
+	for j in range(L):
+		if np.sum(alphas[j]) >= 1 - q:
+			inds = np.argsort(-1*alphas[j])
+			k = np.min(np.where(np.cumsum(alphas[j,inds]) >= 1 - q))
+			group = inds[0:(k+1)].tolist()
+			if purity_threshold > 0 and k + 1 <= k_threshold:
+				mincorr = np.min(np.abs(Sigma[group][:, group]))
+				if mincorr > purity_threshold:
+					groups_to_add.append(group)
+					Linds.append(j)
+			elif k + 1 <= k_threshold:
+				groups_to_add.append(group)
+				Linds.append(j)
+
+	# Adjust alphas
+	if len(Linds) == 0:
+		return []
+	alphas = alphas[Linds, :]
+
+	# sequential groups
 	cand_groups = sequential_groups(
 		susie_alphas=alphas, 
 		q=q,
@@ -184,16 +225,7 @@ def susie_groups(
 	# Add hierarchical groups
 	if X is not None:
 		dist_matrix = np.abs(1 - np.corrcoef(X.T))
-		groups_to_add = _dist_matrices_to_groups(dist_matrix)
-	else:
-		groups_to_add = []
-
-	# Add groups discovered by susie
-	for j in range(L):
-		if np.sum(alphas[j]) >= 1 - q:
-			inds = np.argsort(-1*alphas[j])
-			k = np.min(np.where(np.cumsum(alphas[j,inds]) >= 1 - q))
-			groups_to_add.append(inds[0:(k+1)].tolist())
+		groups_to_add.extend(_dist_matrices_to_groups(dist_matrix))
 
 	# Add these to cand_groups
 	groups_to_add = _dedup_list_of_lists(groups_to_add)
@@ -216,7 +248,7 @@ def sequential_groups(
 	samples=None, 
 	susie_alphas=None,
 	q=0,
-	max_pep=1,
+	max_pep=0.25,
 	max_size=25,
 	prenarrow=False
 ):
@@ -313,7 +345,7 @@ def sequential_groups(
 def hierarchical_groups(
 	samples,
 	dist_matrix=None,
-	max_pep=1,
+	max_pep=0.25,
 	max_size=25,
 	filter_sequential=False,
 	**kwargs
@@ -491,3 +523,62 @@ def _elim_redundant_features(cand_groups):
 	return cand_groups, nrel
 
 
+def _ecc_reduction(cand_groups):
+	"""
+	Given a set of ``CandidateGroup``s with finite group
+	attributes, performs an ECC reduction to reduce
+	the number of locations.
+
+	Parameters
+	----------
+	cand_groups : list
+		A list of CandidateGroup objects.
+
+	Returns
+	-------
+	cand_groups : list
+		The same list of CandidateGroup objects but with
+		an updated 'blip-group' attribute.
+	nrel : int
+		Number of locations in reduced problem.
+
+
+	Notes
+	-----
+	This function is intended as a preprocessing step
+	for certain discrete problems. For continuous problems,
+	use the functions in the ``pyblip.create_groups_cts``
+	module.
+
+	It is not yet efficient and is largely used to reduce very
+	small problems to avoid solver error. 
+	"""
+	hasblipgroup = ['blip-group' in cg.data for cg in cand_groups]
+	if not np.all(np.array(hasblipgroup)):
+		for cg in cand_groups:
+			cg.data['blip-group'] = cg.group
+
+	# O(|G|^2) algorithm to find intersections
+	m = len(cand_groups)
+	G = np.eye(m).astype(bool)
+	for i, cg1 in enumerate(cand_groups):
+		g1 = cg1.data['blip-group']
+		for j in range(i):
+			g2 = cand_groups[j].data['blip-group']
+			if len(g1.intersection(g2)) > 0:
+				G[i, j] = True
+				G[j, i] = True
+
+	# edge clique cover
+	G = nx.Graph(G)
+	cliques = edge_clique_cover(G)
+	# reset blip-group
+	for cg in cand_groups:
+		cg.data['blip-group'] = []
+	for cliquenum, clique in enumerate(cliques):
+		for i in clique:
+			cand_groups[i].data['blip-group'].append(cliquenum)
+	for cg in cand_groups:
+		cg.data['blip-group'] = set(cg.data['blip-group'])
+
+	return cand_groups, len(cliques)
