@@ -1,12 +1,14 @@
 import time
 import numpy as np
 import scipy as sp
+import scipy.linalg
 from scipy import stats
 import unittest
 import pytest
 from .context import pyblip
 from collections import Counter
 from pyblip import create_groups, create_groups_cts
+from pyblip.create_groups import CandidateGroup
 
 class CheckCandGroups(unittest.TestCase):
 	"""
@@ -54,24 +56,53 @@ class CheckCandGroups(unittest.TestCase):
 						decimal=5,
 						err_msg=f"Prop. nsignals={nsignal} at {key} should be {expected_prop} but is {observed_prop}"
 					)
+
+	def check_blip_groups_correct(self, cand_groups):
+		for j1, cg1 in enumerate(cand_groups):
+			g1 = cg1.group
+			bg1 = cg1.data['blip-group']
+			for j2 in range(j1):
+				g2 = cand_groups[j2].group
+				bg2 = cand_groups[j2].data['blip-group']
+				if len(g1.intersection(g2)) > 0:
+					self.assertTrue(
+						len(bg1.intersection(bg2)) > 0,
+						f'Groups {g1}, {g2} overlap but BLiP groups {bg1}, {bg2} do not. cgs={cand_groups}.'
+					)
+				else:
+					self.assertTrue(
+						len(bg1.intersection(bg2)) == 0,
+						f'Groups {g1}, {g2} do not overlap but BLiP groups {bg1}, {bg2} do. cgs={cand_groups}.'
+					)
 		
 	def check_cand_groups_correct(self, cand_groups, shape):
 		if shape != 'circle':
 			raise NotImplementedError("Only works for circles currently")
 		# Check that overlap calculations are correct
-		for cg1 in cand_groups:
+		for j1, cg1 in enumerate(cand_groups):
 			g1 = cg1.group
-			for cg2 in cand_groups:
+			for j2, cg2 in enumerate(cand_groups):
+				if j2 >= j1:
+					continue
 				g2 = cg2.group
+				c1 = np.array(cg1.data['center'])
+				c2 = np.array(cg2.data['center'])
+				dist = np.sqrt(np.sum(np.power(c1-c2, 2)))
+				r1, r2 =  cg1.data['radius'], cg2.data['radius']
 				if len(g1.intersection(g2)) > 0:
-					c1 = np.array(cg1.data['center'])
-					c2 = np.array(cg2.data['center'])
-					dist = np.sqrt(np.sum(np.power(c1-c2, 2)))
-					r1, r2 =  cg1.data['radius'], cg2.data['radius']
 					self.assertTrue(
 						dist < r1 + r2,
 						f"""
-						BLiP posits c1, c2 groups ({g1}, {g2}) overlap. Centers= {c1} and {c2} with dist={dist} and r1={r1} and r2={r2}.
+						BLiP posits c1, c2 groups ({g1}, {g2}) overlap. Centers= {c1} and {c2} with dist={dist}
+						and r1={r1}, r2={r2}, j1={j1}, j2={j2}.
+						"""
+					)
+				else:
+					self.assertTrue(
+						dist >= r1 + r2 + 1e-5,
+						f"""
+						BLiP posits c1, c2 groups ({g1}, {g2}) do not overlap. Centers= {c1} and {c2}  with dist={dist}
+						and r1={r1}, r2={r2}, j1={j1}, j2={j2}.
 						"""
 					)
 
@@ -91,6 +122,30 @@ class TestCandGroups(CheckCandGroups):
 			"_dedup_list_of_lists fails to accurately deduplicate"
 		)
 
+	def test_ecc_reduction(self):
+		cand_groups1 = [
+			CandidateGroup(pep=0, group=set([0,1])),
+			CandidateGroup(pep=0, group=set([1,2])),
+			CandidateGroup(pep=0, group=set([0,2]))
+		]
+		ncg = 100
+		p = 50
+		cand_groups2 = []
+		for j in range(ncg):
+			cand_groups2.append(
+				CandidateGroup(
+					pep=0, 
+					group=set(np.random.choice(np.arange(p), 5))
+				)
+			)
+		for i, cand_groups in enumerate([cand_groups1, cand_groups2]):
+			cand_groups, nrel = pyblip.create_groups._ecc_reduction(cand_groups)
+			self.check_blip_groups_correct(cand_groups)
+			if i == 0:
+				self.assertTrue(
+					nrel==1, f"For simple problem nrel={nrel}, should be 1, cgs={cand_groups}."
+				)
+
 	def test_sequential_groups(self):
 		
 		# Generate fake data
@@ -103,7 +158,7 @@ class TestCandGroups(CheckCandGroups):
 		])
 
 		cand_groups = create_groups.sequential_groups(
-			samples, max_size=4
+			samples, max_size=4, max_pep=1
 		)
 		expected_length = int(5+4+3+2) - 1 # ignores columm with all zeros
 		self.assertTrue(
@@ -190,7 +245,7 @@ class TestCandGroups(CheckCandGroups):
 
 		# Check that the default includes the expected groups
 		cand_groups = create_groups.hierarchical_groups(
-			samples, filter_sequential=False
+			samples, filter_sequential=False, max_pep=1
 		)
 		groups = [x.group for x in cand_groups]
 		expected = [set([i]) for i in [0,1,3,4]] + [set([0,1]), set([0,1,2,3,4])]
@@ -253,6 +308,39 @@ class TestCandGroups(CheckCandGroups):
 				err_msg=f'susie_groups computed the wrong pep for group={g}, pep={cg.pep}, expected={pep}'
 			)
 
+	def test_susie_groups_purity(self):
+		# susie alphas, L = 3, p = 5
+		p = 5
+		n = 10000
+		rho = 0.9
+
+		# Correlation matrix
+		c = np.cumsum(np.zeros(p) + np.log(rho)) - np.log(rho)
+		cov = scipy.linalg.toeplitz(np.exp(c))
+		X = np.dot(np.random.randn(n, p), np.linalg.cholesky(cov).T)
+
+		# susie alphas
+		alphas = np.array([
+			[0.1, 0.5, 0.4, 0.0, 0.0],
+			[0.5, 0.0, 0.0, 0.0, 0.5],
+			[0.2, 0.2, 0.2, 0.2, 0.2],
+		])
+		pt = 0.7
+		cand_groups = create_groups.susie_groups(
+			alphas, 
+			X=X, 
+			purity_threshold=pt,
+			q=0.1,
+			max_pep=1
+		)
+		for cg in cand_groups:
+			expected = 1 - alphas[0, list(cg.group)].sum()
+			np.testing.assert_almost_equal(
+				expected, cg.pep, 3,
+				f"PEP {cg.pep} != expected {expected} for susie group={cg.group} with purity threshold={pt}"
+
+			)
+
 
 
 class TestCtsPEPs(CheckCandGroups):
@@ -338,7 +426,7 @@ class TestCtsPEPs(CheckCandGroups):
 
 		# Compute BLiP nodes
 		all_cgroups, components = create_groups_cts.grid_peps_to_cand_groups(
-			peps, verbose=True, shape='square'
+			peps, verbose=True, shape='circle'
 		)
 		self.assertTrue(
 			len(components) == 1,
@@ -369,7 +457,7 @@ class TestCtsPEPs(CheckCandGroups):
 			[[0.549, 0.451], [0.305, 0.201]],
 			[[0.553, 0.456], [np.nan, np.nan]]
 		])
-		
+
 		# Run with one manual center added
 		xc1 = 0.55012
 		yc1 = 0.45012
